// funkce pro ovladani displeje s obvodem HD44780

#define LCD_Data		PORTB
#define LCD_DataD		DDRB
#define LCD_SetEnable()	PORTB |= 0x08;
#define LCD_ClrEnable() PORTB &=~0x08;
#define LCD_SetData()	PORTC |= 0x02;
#define LCD_SetCommand()PORTC &=~0x02;

void Wait_us(unsigned int val); // 8MHz, optimalization Os
void Wait_ms(unsigned int val);
void LCD_Write(unsigned char val);
void LCD_WriteComm(unsigned char val);
void LCD_WriteData(unsigned char val);
void LCD_Clear(void);
void LCD_Init(void);
void LCD_Position(unsigned char y, unsigned char x);
void LCD_WriteCString(const char* retezec);
void LCD_WriteString(char* retezec);
void LCD_PrDec(unsigned char val); // dva znaky - 8 bitu decimalne
void LCD_PrHex(unsigned char val); // dva znaky - 8 bitu
void LCD_PrHexInt(unsigned int val); // ctyri znaky - 16 bitu
void LCD_PrHex1(unsigned char val); // jeden znak - spodni 4 bity
void LCD_PrSDec(unsigned char val); // tri znaky - 8 bitu decimalne se znamenkem

void LCD_PrSDec(unsigned char val)
{
  if (val&0x80)
  {
    LCD_WriteData('-');
	val = ~val+1;
  }
  else LCD_WriteData(' ');
  LCD_PrDec( val );
}

void LCD_PrHex1(unsigned char val)
{
  val &= 0x0F;
  if (val < 10)
    LCD_WriteData(val+0x30);
  else
    LCD_WriteData(val+0x37);
}

void LCD_PrHexInt(unsigned int val)
{
  unsigned char i;
  i = (unsigned char)(val >> 8);
  LCD_PrHex(i);
  i = (unsigned char)val;
  LCD_PrHex(i);
}

void LCD_PrHex(unsigned char val)
{
  unsigned char znak;
  znak = val / 16;
  if (znak < 10)
    LCD_WriteData(znak+0x30);
  else
    LCD_WriteData(znak+0x37);
  znak = val % 16;
  if (znak < 10)
    LCD_WriteData(znak+0x30);
  else
    LCD_WriteData(znak+0x37);
}

void LCD_PrDec(unsigned char val)
{
  LCD_WriteData((val / 10) + 0x30);
  LCD_WriteData((val % 10) + 0x30);
}

void LCD_WriteString(char* retezec)
{
  while(*retezec != '\0')
  {
    LCD_WriteData(*retezec);
	retezec +=1;
  }
}

void LCD_WriteCString(const char* retezec)
{
  while(*retezec != '\0')
  {
    LCD_WriteData(*retezec);
	retezec +=1;
  }
}

void LCD_Position(unsigned char y, unsigned char x)
{
  switch(y)
  {
    case 0: x+=0x00; break;
	case 1: x+=0x40; break;
	case 2: x+=0x14; break;
	case 3: x+=0x54; break;
  }
  LCD_WriteComm(x|0x80);
}

void LCD_Init(void)
{
  LCD_ClrEnable();
  LCD_Clear();
  LCD_WriteComm(0x28);
  LCD_WriteComm(0x08);
  LCD_Clear();
  LCD_WriteComm(0x06);
  LCD_WriteComm(0x0C);
}

void LCD_Clear(void)
{
  LCD_WriteComm(1);
  Wait_ms(100);
}

void LCD_WriteData(unsigned char val)
{
  LCD_SetData();
  LCD_Write(val);
}

void LCD_WriteComm(unsigned char val)
{
  LCD_SetCommand();
  LCD_Write(val);
}

void LCD_Write(unsigned char val)
{
  LCD_Data = (LCD_Data & 0x0F) | (val & 0xF0);
  LCD_SetEnable();
  asm("wdr"); // nuluje watchdog
  LCD_ClrEnable();
  LCD_Data = (LCD_Data & 0x0F) | (val << 4);
  LCD_SetEnable();
  asm("wdr"); // nuluje watchdog
  LCD_ClrEnable();
  Wait_us(100);
}

void Wait_ms(unsigned int val)
{
  while(val--)
  {
    Wait_us(1000);
  }
}

void Wait_us(unsigned int val) // 8MHz, optimalization Os
{
  while(val--)
  {
    EECR |= 0x00;
  }
}

